# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test signals."""

from __future__ import absolute_import, print_function

from invenio_base.app import create_app_factory
from invenio_base.signals import app_created, app_loaded


def test_create_app_factory():
    """Test signals sending."""
    calls = {'created': 0, 'loaded': 0}
    create_app = create_app_factory('test')

    def _receiver(name):
        def _inner(sender, app=None):
            calls[name] += 1
            calls['{}_app'.format(name)] = app
        return _inner

    app_created.connect(_receiver('created'), sender=create_app, weak=False)
    app_loaded.connect(_receiver('loaded'), sender=create_app, weak=False)

    assert callable(create_app)

    app = create_app()
    assert calls['created'] == 1
    assert calls['loaded'] == 1
    assert calls['created_app'] is app
    assert calls['loaded_app'] is app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

from __future__ import absolute_import, print_function

import shutil
import tempfile

import pytest


@pytest.fixture
def tmppath(request):
    """Application fixture."""
    tmppath = tempfile.mkdtemp()

    def teardown():
        shutil.rmtree(tmppath)

    request.addfinalizer(teardown)
    return tmppath



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Signals for application creation."""

from blinker import Namespace

_signals = Namespace()

app_created = _signals.signal('app-created')
"""Signal sent when the base Flask application have been created.

Parameters:
- ``sender`` - the application factory function.
- ``app`` - the Flask application instance.

Example receiver:

.. code-block:: python

   def receiver(sender, app=None, **kwargs):
       # ...
"""

app_loaded = _signals.signal('app-loaded')
"""Signal sent when the Flask application have been fully loaded.

Parameters:
- ``sender`` - the application factory function.
- ``app`` - the Flask application instance.

Example receiver:

.. code-block:: python

   def receiver(sender, app=None, **kwargs):
       # ...
"""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test utilities."""

from __future__ import absolute_import, print_function

from flask import Flask

from invenio_base.utils import load_or_import_from_config, obj_or_import_string
from invenio_base.wsgi import wsgi_proxyfix


def test_obj_or_import_string():
    """Test object or string import utility function."""
    assert obj_or_import_string(wsgi_proxyfix) == wsgi_proxyfix
    assert obj_or_import_string(None, default=wsgi_proxyfix) == wsgi_proxyfix
    assert obj_or_import_string(
        'invenio_base.wsgi.wsgi_proxyfix') == wsgi_proxyfix


def test_load_or_import_from_config():
    """Test load or import from config utility function."""
    app = Flask('testapp')
    app.config['OBJ_KEY'] = wsgi_proxyfix
    app.config['PATH_KEY'] = 'invenio_base.wsgi.wsgi_proxyfix'

    with app.app_context():
        assert load_or_import_from_config('OBJ_KEY') == wsgi_proxyfix
        assert load_or_import_from_config('PATH_KEY') == wsgi_proxyfix
        assert load_or_import_from_config(
            'MISSING_KEY', default=wsgi_proxyfix) == wsgi_proxyfix



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test wsgi application."""

from __future__ import absolute_import, print_function

import json

import pytest
from flask import Flask, jsonify, request
from packaging import version
from werkzeug import __version__ as werkzeug_version

from invenio_base.wsgi import create_wsgi_factory, wsgi_proxyfix


def test_create_wsgi_factory():
    """Test wsgi factory creation."""
    api = Flask('api')

    @api.route('/')
    def apiview():
        return 'api'

    app = Flask('app')

    @app.route('/')
    def appview():
        return 'app'

    # Test factory creation
    factory = create_wsgi_factory({'/api': lambda **kwargs: api})
    app.wsgi_app = factory(app)

    with app.test_client() as client:
        assert client.get('/').status_code == 200
        assert b'app' in client.get('/').data
        assert client.get('/api/').status_code == 200
        assert b'api' in client.get('/api/').data


@pytest.mark.parametrize('proxies,data', [
    (2, b'4.3.2.1'), (None, b'1.2.3.4')
])
def test_proxyfix_wsgi_proxies(proxies, data):
    """Test wsgi factory creation."""
    app = Flask('app')
    app.config['WSGI_PROXIES'] = proxies

    @app.route('/')
    def appview():
        return str(request.remote_addr)

    # Test factory creation
    app.wsgi_app = wsgi_proxyfix()(app)
    e = {'REMOTE_ADDR': '1.2.3.4'}

    with app.test_client() as client:
        h = {'X-Forwarded-For': '5.6.7.8, 4.3.2.1, 8.7.6.5'}
        assert client.get('/', headers=h, environ_base=e).data == data


@pytest.mark.parametrize(
    'num_proxies,proxy_config', [
        (n, {'x_for': n, 'x_proto': n, 'x_host': n, 'x_port': n,
             'x_prefix': n}) for n in range(2)])
def test_proxyfix_wsgi_config(num_proxies, proxy_config):
    """Test wsgi factory creation with APP_WSGI_CONFIG set."""
    if version.parse(werkzeug_version) < version.parse('0.15.0'):
        pytest.skip("Unsupported configuration for Werkzeug<0.15.0")

    app = Flask('app')
    app.config['PROXYFIX_CONFIG'] = proxy_config

    data = [
        # application instance
        {
            'x_for': '1.2.3.4',
            'x_proto': 'http',
            'x_host': 'localhost',
            'x_port': '80',
            'x_prefix': '',
        },
        # proxy number 1
        {
            'x_for': '5.6.7.8',
            'x_proto': 'https',
            'x_host': 'host.external',
            'x_port': '443',
            'x_prefix': 'prefix.external',
        }
    ]

    @app.route('/')
    def appview():
        data = {
            'x_for': request.environ.get('REMOTE_ADDR'),
            'x_proto': request.environ.get('wsgi.url_scheme'),
            'x_host': request.environ.get('SERVER_NAME'),
            'x_port': request.environ.get('SERVER_PORT'),
            'x_prefix': request.environ.get('SCRIPT_NAME')
        }
        return jsonify(data)

    # Test factory creation
    app.wsgi_app = wsgi_proxyfix()(app)
    e = {
        'REMOTE_ADDR': data[0].get('x_for'),
        'wsgi.url_scheme': data[0].get('x_proto'),
        'HTTP_HOST': data[0].get('x_host'),
        'SERVER_PORT': data[0].get('x_port'),
        'SCRIPT_NAME': data[0].get('x_prefix'),
    }

    with app.test_client() as client:
        h = {
            'X-Forwarded-For': data[1].get('x_for'),
            'X-Forwarded-Proto': data[1].get('x_proto'),
            'X-Forwarded-Host': data[1].get('x_host'),
            'X-Forwarded-Port': data[1].get('x_port'),
            'X-Forwarded-Prefix': data[1].get('x_prefix'),
        }
        res = client.get('/', headers=h, environ_base=e)
        assert json.loads(res.get_data(as_text=True)) == data[num_proxies]



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Application bootstraping."""

from __future__ import absolute_import, print_function

import warnings

import click
from flask import current_app
from flask.cli import with_appcontext
from pkg_resources import iter_entry_points, resource_filename, working_set


@click.group()
def instance():
    """Instance commands."""


@instance.command('entrypoints')
@click.option(
    '-e', '--entry-point', default=None, metavar='ENTRY_POINT',
    help='Entry point group name (e.g. invenio_base.apps)')
def list_entrypoints(entry_point):
    """List defined entry points."""
    found_entry_points = {}
    for dist in working_set:
        entry_map = dist.get_entry_map()
        for group_name, entry_points in entry_map.items():
            # Filter entry points
            if entry_point is None and \
               not group_name.startswith('invenio'):
                continue
            if entry_point is not None and \
               entry_point != group_name:
                continue

            # Store entry points.
            if group_name not in found_entry_points:
                found_entry_points[group_name] = []
            for ep in entry_points.values():
                found_entry_points[group_name].append(str(ep))

    for ep_group in sorted(found_entry_points.keys()):
        click.secho('{0}'.format(ep_group), fg='green')
        for ep in sorted(found_entry_points[ep_group]):
            click.echo('  {0}'.format(ep))


@instance.command('migrate-secret-key')
@click.option('--old-key', required=True)
@with_appcontext
def migrate_secret_key(old_key):
    """Call entry points exposed for the SECRET_KEY change."""
    if 'SECRET_KEY' not in current_app.config or \
            current_app.config['SECRET_KEY'] is None:
        raise click.ClickException(
            'SECRET_KEY is not set in the configuration.')

    migrators = []
    for ep in iter_entry_points('invenio_base.secret_key'):
        try:
            migrators.append(ep.load())
        except Exception:
            raise click.ClickException(
                'Failed to initialize entry point: {0}'.format(ep)
            )

    for m in migrators:
        try:
            m(old_key=old_key)
        except Exception:
            raise click.ClickException(
                'Failed to perform migration of secret key {0}'.format(old_key)
            )

    click.secho('Successfully changed secret key.', fg='green')


def generate_secret_key():
    """Generate secret key."""
    import random
    import string

    rng = random.SystemRandom()
    return ''.join(
        rng.choice(string.ascii_letters + string.digits)
        for dummy in range(0, 256)
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Base utilities."""

import six
from flask import current_app
from werkzeug.utils import import_string


def obj_or_import_string(value, default=None):
    """Import string or return object.

    :params value: Import path or class object to instantiate.
    :params default: Default object to return if the import fails.
    :returns: The imported object.
    """
    if isinstance(value, six.string_types):
        return import_string(value)
    elif value:
        return value
    return default


def load_or_import_from_config(key, app=None, default=None):
    """Load or import value from the application config.

    :params key: Configuration key.
    :params app: Flask application (by default ``flask.current_app``).
    :params default: Default object to return if the import fails
        (by default ``None``).
    :returns: The loaded value.
    """
    app = app or current_app
    value = app.config.get(key)
    return obj_or_import_string(value, default=default)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio application loader.

Quickstart
----------
Invenio-Base is taking advantage of advanced patterns for building Flask
application. It assumes you already have understanding of
`patterns for Flask <http://flask.pocoo.org/docs/dev/patterns/>`_.

Dependencies
~~~~~~~~~~~~
First we need to install and import dependencies:

.. code-block:: console

   $ mkvirtualenv example
   (example)$ pip install invenio-base

Now you can create new file ``app.py`` with following imports:

.. include:: ../examples/app.py
   :start-after: # sphinxdoc-example-import-begin
   :end-before: # sphinxdoc-example-import-end
   :literal:

Configuration
~~~~~~~~~~~~~
Tell the application factory how to load configuration by creating
``config_loader`` function that accepts an application instance:

.. include:: ../examples/app.py
   :start-after: # sphinxdoc-example-config-begin
   :end-before: # sphinxdoc-example-config-end
   :literal:

The recommended way is to use `Invenio-Config
<https://invenio-config.readthedocs.io/>`_ that provides a default
configuration loader :func:`invenio_config.utils.create_config_loader` which is
sufficient for most cases:

.. code-block:: python

   from invenio_config import create_config_loader
   config_loader = create_config_loader(config=Config, env_prefix='APP')

In the next step you should set an absolute path for the *instance folder* in
order to load configuration files and other data from deployment specific
location. The instance folder is also perfect place for dropping static files
if you do not serve them from CDN:

.. include:: ../examples/app.py
   :start-after: # sphinxdoc-example-paths-begin
   :end-before: # sphinxdoc-example-paths-end
   :literal:

In our example the variables are read from environment variables first with the
purpose that they can be easily changed without modifying code for various
deployment usecases.

Combining Applications
~~~~~~~~~~~~~~~~~~~~~~
It is highly recommendended to separate Invenio UI and REST applications then
different exception handlers, URL converters and session management can be
installed on each application instance. You can even install your own WSGI
application side by side with Invenio ones.

Invenio packages provide apps (extensions), blueprints, and URL converters via
entry points ``invenio_base.[api_]<apps,blueprints,converters>``. You can
specify multiple entry point groups for each application factory (e.g.
``myservice.blueprints``):

.. include:: ../examples/app.py
   :start-after: # sphinxdoc-example-factories-begin
   :end-before: # sphinxdoc-example-factories-end
   :literal:

You provide instances of your own apps, blueprints, or URL converters directly
to the factory:

.. code-block:: python

   from flask import Blueprint

   blueprint = Blueprint('example', __name__)

   @blueprint.route('/')
   def index():
       return 'Hello from Example application.'

    create_app = create_app_factory(
        'example',
        blueprints=[blueprint],
        # other parameters as shown in previous example
    )

Running
~~~~~~~
To run you application you need to first instantiate the application object:

.. include:: ../examples/app.py
   :start-after: # sphinxdoc-example-objects-begin
   :end-before: # sphinxdoc-example-objects-end
   :literal:

Then you need to tell the **``flask``** command where is your file located
by setting environment variable ``FLASK_APP=app.py``:

.. code-block:: console

   $ export FLASK_APP=app.py
   $ flask run

If you prefer to make your own executable script then you can use following
pattern:

.. code-block:: python

   from invenio_base.app import create_cli

   cli = create_cli(create_app=create_app)

   if __name__ == '__main__':
       cli()

Do not worry, you do not have to write all this by yourself. Follow next steps
and use ``inveniomanage`` command that generates the scaffold code for you.

The ``inveniomanage`` command
-----------------------------
Invenio-Base installs the ``inveniomanage`` command. By default only three
subcommands are available:

.. code-block:: console

   $ inveniomanage --help
   Usage: inveniomanage [OPTIONS] COMMAND [ARGS]...

     Command Line Interface for Invenio.

   Options:
     -a, --app TEXT        The application to run.
     --debug / --no-debug  Enable or disable debug mode.
     --help                Show this message and exit.

   Commands:
     run              Run development server.
     shell            Run shell in the app context.

The ``run`` and ``shell`` commands only works if you have specified the
``--app`` option or the ``FLASK_APP`` environment variable. See
`Flask <http://flask.pocoo.org/docs/dev/cli/>`_ documentation for further
information.


Listing all entrypoints of an Invenio instance
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
The ``instance entrypoints`` subcommand helps you list all entrypoints of your
Invenio application:

.. code-block:: console

   $ inveniomanage instance entrypoints

The output of the command will be in the below format:

.. code-block:: console

   <entrypoint_group_name>
     <entrypoint>


You can also restrict the output of the command to list all entrypoints for a
specific entrypoint group by passing the name via the `-e` option:

.. code-block:: console

   $ inveniomanage instance entrypoints -e <entrypoint_group_name>

For further details about the available options run the `help` command:

.. code-block:: console

   $ inveniomanage instance entrypoints --help
   ...


Migrating the application's old secret key
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
The ``instance migrate_secret_key`` subcommand helps you migrate your
application's old secret key:

.. code-block:: console

   $ inveniomanage instance migrate_secret_key --old-key <old_key>

The purpose of this command is to provide the administrator the capability to
change the Invenio application's secret_key and migrate that change in all
database's EncryptedType properties through an entrypoint group called
`invenio_base.secret_key'`. There you can specify your migration function that
will receive the old secret_key that can be used to decrypt the old properties
and encrypt them again with the application's new secret_key.

You can register your migration function as shown below in your package's
entrypoints in the setup.py:

.. code-block:: console

   entrypoints= {
       'invenio_base.secret_key': [
           '<entrypoint_name> = <entrypoint_function>'
       ]
   }

Also you can see an example of use in `invenio_oauthclient
<https://github.com/inveniosoftware/invenio-oauthclient>`_
package's setup.py.

.. note::
 You should change your application's `secret_key` in the config before calling
 the migration command.

For further details about the available options run the `help` command:

.. code-block:: console

   $ inveniomanage instance migrate_secret_key --help
   ...
"""

from __future__ import absolute_import, print_function

from .app import create_app_factory, create_cli
from .version import __version__
from .wsgi import create_wsgi_factory

__all__ = (
    '__version__',
    'create_app_factory',
    'create_cli',
    'create_wsgi_factory',
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test basic application."""

from __future__ import absolute_import, print_function

import logging
import warnings
from os.path import exists, join

import click
import pytest
from click.testing import CliRunner
from flask import Blueprint, Flask, current_app
from mock import patch
from pkg_resources import EntryPoint
from werkzeug.routing import BaseConverter

from invenio_base import __version__
from invenio_base.app import _loader, app_loader, base_app, blueprint_loader, \
    configure_warnings, converter_loader, create_app_factory, create_cli
from invenio_base.cli import generate_secret_key

try:
    from werkzeug.middleware.dispatcher import DispatcherMiddleware
except ImportError:
    from werkzeug.wsgi import DispatcherMiddleware


class ListConverter(BaseConverter):
    """Simple list converter."""

    def to_python(self, value):
        """Return Python object."""
        return value.split('+')

    def to_url(self, values):
        """Return string."""
        return '+'.join(BaseConverter.to_url(value)
                        for value in values)


#
# Mock helpers
#
class MockLoggingHandler(logging.Handler):
    """Mock logging handler to check for expected logs."""

    def __init__(self, *args, **kwargs):
        """Initialize handler."""
        self.reset()
        logging.Handler.__init__(self, *args, **kwargs)

    def emit(self, record):
        """Emit log record by saving message to internal list."""
        self.messages[record.levelname.lower()].append(record.getMessage())

    def reset(self):
        """Reset internal list of messages."""
        self.messages = {
            'debug': [],
            'info': [],
            'warning': [],
            'error': [],
            'critical': [],
        }


class MockEntryPoint(EntryPoint):
    """Mocking of entrypoint."""

    def load(self):
        """Mock load entry point."""
        if self.name == 'fail':
            raise Exception("Fail")
        return self.name


class NoRequireEntryPoint(EntryPoint):
    """Load without requirements check."""

    def load(self):
        """Mock load entry point."""
        return super(NoRequireEntryPoint, self).resolve()


def _mock_entry_points(name):
    data = dict(
        entrypoint1=[MockEntryPoint('ep1.e1', 'ep1.e1'),
                     MockEntryPoint('ep1.e2', 'ep1.e2'), ],
        entrypoint2=[MockEntryPoint('ep2.e1', 'ep2.e1'),
                     MockEntryPoint('ep2.e2', 'ep2.e2'), ],
        entrypoint3=[MockEntryPoint('fail', 'ep3.e1',), ],
        entrypoint4=[NoRequireEntryPoint.parse(
            'mylist = test_app:ListConverter'), ],
    )
    names = data.keys() if name is None else [name]
    for key in names:
        for entry_point in data[key]:
            yield entry_point


#
# Tests
#
def test_version():
    """Test version."""
    assert __version__


def test_configure_warnings():
    """Test warnings configuration."""
    logger = logging.getLogger('py.warnings')
    handler = MockLoggingHandler()
    logger.addHandler(handler)

    # Warnings not routed through logging
    warnings.warn("Test")
    assert handler.messages['warning'] == []

    # Warnings through logging
    configure_warnings()
    warnings.warn("A warning")
    assert "A warning" in handler.messages['warning'][0]

    handler.reset()
    warnings.warn("Pending deprecation", PendingDeprecationWarning)
    warnings.warn("Deprecation", DeprecationWarning)
    assert len(handler.messages['warning']) == 1

    handler.reset()
    warnings.simplefilter("always")
    warnings.warn("Pending deprecation", PendingDeprecationWarning)
    warnings.warn("Deprecation", DeprecationWarning)
    assert len(handler.messages['warning']) == 2
    warnings.resetwarnings()


@patch('pkg_resources.iter_entry_points', _mock_entry_points)
def test_loader():
    """Test loader."""
    app = Flask(__name__)
    found = []
    _loader(app, lambda x: found.append(x), entry_points=None, modules=None)
    assert found == []

    # Modules
    found = []
    _loader(app, lambda x: found.append(x), modules=['a', 'b'])
    assert found == ['a', 'b']

    # Entry points
    found = []
    _loader(
        app,
        lambda x: found.append(x), entry_points=['entrypoint1', 'entrypoint2'])
    assert found == ['ep1.e1', 'ep1.e2', 'ep2.e1', 'ep2.e2']

    # Modules and entry points (entry points loaded before modules)
    found = []
    _loader(
        app,
        lambda x: found.append(x),
        entry_points=['entrypoint1', 'entrypoint2'],
        modules=['a', 'b']
    )
    assert found == ['ep1.e1', 'ep1.e2', 'ep2.e1', 'ep2.e2', 'a', 'b']


@patch('pkg_resources.iter_entry_points', _mock_entry_points)
def test_loader_exceptions():
    """Test exceptions during loading."""
    app = Flask(__name__)
    handler = MockLoggingHandler()
    app.logger.addHandler(handler)

    def _raise_func():
        raise Exception()

    assert len(handler.messages['error']) == 0
    pytest.raises(
        Exception, _loader, app, lambda x: x(),
        entry_points=None, modules=[_raise_func])
    assert len(handler.messages['error']) == 1

    pytest.raises(
        Exception, _loader, app, lambda x: x(),
        entry_points=['entrypoint3'], modules=None)
    assert len(handler.messages['error']) == 2


def test_app_loader():
    """Test app loader."""
    class FlaskExt(object):
        def __init__(self, app):
            self.app = app
            app.extensions['ext'] = self

    app = Flask('testapp')

    assert 'ext' not in app.extensions
    app_loader(app, modules=[FlaskExt])
    assert 'ext' in app.extensions
    assert app.extensions['ext'].app is app


def test_blueprint_loader():
    """Test app loader."""
    bp = Blueprint('test', 'test')

    def create_blueprint_func(app):
        return Blueprint('test2', 'test2')

    app = Flask('testapp')

    assert len(app.blueprints) == 0
    blueprint_loader(app, modules=[bp, create_blueprint_func])
    assert len(app.blueprints) == 2
    assert 'test' in app.blueprints
    assert 'test2' in app.blueprints


def test_coverter_loader():
    """Test converter loader."""
    app = Flask('testapp')

    assert 'mylist' not in app.url_map.converters
    converter_loader(app, modules={'mylist': ListConverter})
    assert 'mylist' in app.url_map.converters


@patch('pkg_resources.iter_entry_points', _mock_entry_points)
def test_coverter_loader_from_entry_points():
    """Test converter loader."""
    app = Flask('testapp')

    assert 'mylist' not in app.url_map.converters
    converter_loader(app, entry_points=['entrypoint4'])
    assert 'mylist' in app.url_map.converters


@patch('pkg_resources.iter_entry_points', _mock_entry_points)
def test_coverter_loader_fail():
    """Test converter loader."""
    app = Flask('testapp')

    with pytest.raises(Exception):
        converter_loader(app, entry_points=['entrypoint3'])


def test_base_app(tmppath):
    """Test base app creation."""
    # Test default static_url_path and CLI initialization
    app = base_app('test')
    assert app.name == 'test'
    assert app.cli
    assert app.static_url_path == '/static'
    assert app.instance_path != tmppath

    # Test specifying instance path
    app = base_app('test', instance_path=tmppath)
    assert app.instance_path == tmppath
    assert exists(app.instance_path)
    assert app.static_folder is None

    # Test automatic instance path creation
    newpath = join(tmppath, 'test')
    assert not exists(newpath)
    app = base_app('test', instance_path=newpath)
    assert exists(newpath)
    assert app.static_folder is None

    # Test static folder
    staticpath = join(tmppath, 'teststatic')
    app = base_app('test', static_folder=staticpath)
    assert app.static_folder == staticpath
    assert app.instance_path is not None

    # Test static + instance folder
    staticpath = join(tmppath, 'teststatic')
    app = base_app('test', instance_path=tmppath,
                   static_folder=staticpath)
    assert app.static_folder == staticpath
    assert app.instance_path == tmppath

    # Test choice loader
    searchpath = join(tmppath, "tpls")
    app = base_app('test', template_folder=searchpath)
    assert app.jinja_loader.searchpath == [searchpath]

    app = base_app('test')
    assert app.jinja_loader.searchpath == [join(app.root_path, 'templates')]


def test_base_app_class(tmppath):
    """Test using custom Flask application class."""
    class CustomFlask(Flask):
        pass

    app = base_app('test', app_class=CustomFlask)
    assert isinstance(app, CustomFlask)


def test_create_app_factory():
    """Test app factory factory."""
    class FlaskExt(object):
        def __init__(self, app):
            self.app = app
            app.extensions['ext'] = self

    bp = Blueprint('test', 'test')

    # Create app
    create_app = create_app_factory(
        'test', blueprints=[bp], extensions=[FlaskExt])
    assert callable(create_app)

    app = create_app()
    assert app.name == 'test'
    assert len(app.blueprints) == 1
    assert 'ext' in app.extensions


def test_create_app_debug_flag():
    """Test debug flag propagation (needed by CLI)."""
    create_app = create_app_factory('test')

    assert create_app().debug is False
    assert create_app(debug=True).debug is True


def test_callable_instance_path(tmppath):
    """Test instance path evaluation."""

    def instance_path():
        return tmppath

    app = create_app_factory('test', instance_path=instance_path)()
    assert app.instance_path == tmppath
    assert app.static_folder is None


def test_callable_static_path(tmppath):
    """Test static path evaluation."""

    def static_folder():
        return join(tmppath, 'teststatic')

    app = create_app_factory('test', static_folder=static_folder)()
    assert app.static_folder == join(tmppath, 'teststatic')
    assert app.instance_path is not None


def test_create_app_factory_config_loader():
    """Test app factory conf loader."""
    def _config_loader(app, **kwargs):
        app.config['CONFIG_LOADER'] = True
        app.config.update(kwargs)

    create_app = create_app_factory('test', config_loader=_config_loader)
    app = create_app(KWARGS_TEST=True)
    assert app.config['CONFIG_LOADER']
    assert app.config['KWARGS_TEST']


def test_create_app_factory_wsgi_factory():
    """Test app factory wsgi factory."""
    def _wsgi_factory(app):
        return DispatcherMiddleware(app.wsgi_app, {'/test': Flask('dispatch')})

    create_app = create_app_factory('test', wsgi_factory=_wsgi_factory)
    app = create_app()
    assert isinstance(app.wsgi_app, DispatcherMiddleware)


def test_create_cli_with_app():
    """Test create cli."""
    app_name = 'mycmdtest'
    create_app = create_app_factory(app_name)
    cli = create_cli(create_app=create_app)

    @cli.command()
    def test_cmd():
        click.echo('{0} {1}'.format(current_app.name, current_app.debug))

    runner = CliRunner()
    result = runner.invoke(cli)
    assert result.exit_code == 0

    result = runner.invoke(cli, ['test-cmd'])
    assert result.exit_code == 0
    assert u'{0} False\n'.format(app_name) in result.output

    # FIXME wait for fixed support in Flask.
    # result = runner.invoke(cli, ['--debug', 'test_cmd'])
    # assert result.exit_code == 0
    # assert u'{0} True\n'.format(app_name) in result.output


def test_create_cli_without_app():
    """Test create cli."""
    from invenio_base.__main__ import cli

    @cli.command()
    def test_cmd():
        click.echo(current_app.name)

    runner = CliRunner()
    result = runner.invoke(cli)
    assert result.exit_code == 0

    result = runner.invoke(cli, ['test-cmd'])
    assert result.exit_code != 0
    assert 'FLASK_APP' in result.output


def test_generate_secret_key():
    """Test generation of a secret key."""
    v1 = generate_secret_key()
    v2 = generate_secret_key()
    assert len(v1) == len(v2) == 256
    assert v1 != v2



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test cli application."""

from __future__ import absolute_import, print_function

import logging
import os
from subprocess import call

import pkg_resources
import pytest
from click.testing import CliRunner
from flask.cli import ScriptInfo
from mock import MagicMock, Mock, patch

from invenio_base.app import create_app_factory
from invenio_base.cli import instance


@pytest.fixture(autouse=True)
def force_logging():
    """Flask 0.13/1.0 changed logging to not add the default
    handler in case a handler is already installed. pytest
    automatically adds a handler to the root logger, causing
    Flask not to add a handler. This is an issue when testing
    Click output which uses the logger to output to the console.
    """
    try:
        from flask.logging import default_handler
        logger = logging.getLogger('flask.app')
        if default_handler not in logger.handlers:
            logger.handlers.append(default_handler)
    except Exception:
        pass


def test_list_entry_points():
    """Test listing of entry points."""
    mock_working_set = pkg_resources.WorkingSet(entries=[])
    dist = pkg_resources.get_distribution('invenio-base')
    mock_working_set.add(dist)

    with patch('invenio_base.cli.working_set', new=mock_working_set):
        runner = CliRunner()

        # Test select an existing entry point
        result = runner.invoke(
            instance, ['entrypoints', '-e', 'console_scripts'])
        assert result.exit_code == 0
        lines = result.output.splitlines()
        assert lines[0] == 'console_scripts'
        assert lines[1] == '  inveniomanage = invenio_base.__main__:cli'

        # Test no entry point matching
        result = runner.invoke(instance, ['entrypoints', '-e', 'nothing_here'])
        assert result.exit_code == 0
        assert result.output == ""

        # By default we only show entry points groups starting with "invenio"
        dist.get_entry_map = Mock(
            return_value={
                'invenio_base.apps': {
                    'myapp': 'myapp = myapp:MyApp',
                    'app1': 'app1 = app1:MyApp'
                },
                'invenio_base.api_apps': {
                    'myapi': 'myapi = myapi:MyApp'
                },
                'console_scripts': {
                    'mycli': 'mycli = cli:main'
                },
            })
        result = runner.invoke(instance, [
            'entrypoints',
        ])
        assert result.exit_code == 0
        print(result.output.splitlines())
        lines = result.output.splitlines()
        assert lines[0] == 'invenio_base.api_apps'
        assert lines[1] == '  myapi = myapi:MyApp'
        assert lines[2] == 'invenio_base.apps'
        assert lines[3] == '  app1 = app1:MyApp'
        assert lines[4] == '  myapp = myapp:MyApp'


def test_migrate_secret_key():
    """Test cli command for SECRET_KEY change."""
    def _config_loader(app, **kwargs):
        app.config['CONFIG_LOADER'] = True
        app.config.update(kwargs)

    create_app = create_app_factory('test', config_loader=_config_loader)
    app = create_app(KWARGS_TEST=True)

    script_info = ScriptInfo(create_app=lambda info: app)

    # Check that CLI command fails when the SECRET_KEY is not set.
    with app.app_context():
        runner = CliRunner()
        result = runner.invoke(
            instance, ['migrate-secret-key', '--old-key', 'OLD_SECRET_KEY'],
            obj=script_info)
        assert result.exit_code == 1
        assert 'Error: SECRET_KEY is not set in the configuration.' in \
            result.output

    app.secret_key = "SECRET"
    with patch('pkg_resources.EntryPoint') as MockEntryPoint:
        # Test that the CLI command succeeds when the entrypoint does
        # return a function.
        entrypoint = MockEntryPoint('ep1', 'ep1')
        entrypoint.load.return_value = MagicMock()
        with patch('invenio_base.cli.iter_entry_points',
                   return_value=[entrypoint]):
            result = runner.invoke(
                instance, ['migrate-secret-key', '--old-key',
                           'OLD_SECRET_KEY'],
                obj=script_info)
            assert result.exit_code == 0
            assert entrypoint.load.called
            entrypoint.load.return_value.assert_called_with(
                old_key='OLD_SECRET_KEY'
            )
            assert 'Successfully changed secret key.' in result.output

        # Test that the CLI command fails correctly when the entrypoint does
        # not return a function.
        entrypoint = MockEntryPoint('ep2', 'ep2')
        entrypoint.load.return_value = 'ep2'
        with patch('invenio_base.cli.iter_entry_points',
                   return_value=[entrypoint]):
            result = runner.invoke(
                instance, ['migrate-secret-key', '--old-key',
                           'OLD_SECRET_KEY'],
                obj=script_info)
            assert result.exit_code == 1
            assert entrypoint.load.called
            assert 'Failed to perform migration of secret key' in result.output



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Base package for building Invenio application factories."""

import os

from setuptools import find_packages, setup

readme = open('README.rst').read()
history = open('CHANGES.rst').read()


# Get the version string. Cannot be done with import!
g = {}
with open(os.path.join('invenio_base', 'version.py'), 'rt') as fp:
    exec(fp.read(), g)
    version = g['__version__']


tests_require = [
    'mock>=1.3.0',
    'invenio-config>=1.0.0',
    'pytest-invenio>=1.4.0'
]

extras_require = {
    'docs': [
        'Sphinx>=3',
    ],
    'tests': tests_require,
}

extras_require['all'] = []
for reqs in extras_require.values():
    extras_require['all'].extend(reqs)

setup_requires = [
    'pytest-runner>=2.6.2',
]

install_requires = [
    'blinker>=1.4',
    'Flask>=1.0.4,<2.0',
    'Werkzeug>=1.0.1,<2.0',
    'six>=1.12.0',
]

packages = find_packages()

setup(
    name='invenio-base',
    version=version,
    description=__doc__,
    long_description=readme + '\n\n' + history,
    keywords='invenio',
    license='MIT',
    author='CERN',
    author_email='info@inveniosoftware.org',
    url='https://github.com/inveniosoftware/invenio-base',
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'console_scripts': [
            'inveniomanage = invenio_base.__main__:cli',
        ],
        'flask.commands': [
            'instance = invenio_base.cli:instance',
        ],
    },
    extras_require=extras_require,
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Development Status :: 5 - Production/Stable',
    ],
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio application factory."""

from __future__ import absolute_import, print_function

import logging
import os.path
import sys
import warnings

import click
import pkg_resources
from flask import Flask
from flask.cli import FlaskGroup
from flask.helpers import get_debug_flag

from .signals import app_created, app_loaded


def create_app_factory(app_name, config_loader=None,
                       extension_entry_points=None, extensions=None,
                       blueprint_entry_points=None, blueprints=None,
                       converter_entry_points=None, converters=None,
                       wsgi_factory=None, **app_kwargs):
    """Create a Flask application factory.

    The application factory will load Flask extensions and blueprints specified
    using both entry points and directly in the arguments. Loading order of
    entry points are not guaranteed and can happen in any order.

    :param app_name: Flask application name.
    :param config_loader: Callable which will be invoked on application
        creation in order to load the Flask configuration. See example below.
    :param extension_entry_points: List of entry points, which specifies Flask
        extensions that will be initialized only by passing in the Flask
        application object
    :param extensions: List of Flask extensions that can be initialized only by
        passing in the Flask application object.
    :param blueprint_entry_points: List of entry points, which specifies
        Blueprints that will be registered on the Flask application.
    :param blueprints: List of Blueprints that will be registered on the
        Flask application.
    :param converter_entry_points: List of entry points, which specifies
        Werkzeug URL map converters that will be added to
        ``app.url_map.converters``.
    :param converters: Map of Werkzeug URL map converter classes that will
        be added to ``app.url_map.converters``.
    :param wsgi_factory: A callable that will be passed the Flask application
        object in order to overwrite the default WSGI application (e.g. to
        install ``DispatcherMiddleware``).
    :param app_kwargs: Keyword arguments passed to :py:meth:`base_app`.
        `instance_path` and `static_folder` can be passed as callables.
    :returns: Flask application factory.

    Example of a configuration loader:

    .. code-block:: python

       def my_config_loader(app, **kwargs):
           app.config.from_module('mysite.config')
           app.config.update(**kwargs)

    .. note::

       `Invenio-Config <https://pythonhosted.org/invenio-config>`_ provides a
       factory creating default configuration loader (see
       :func:`invenio_config.utils.create_config_loader`) which is sufficient
       for most cases.

    Example of a WSGI factory:

    .. code-block:: python

       def my_wsgi_factory(app):
           return DispatcherMiddleware(app.wsgi_app, {'/api': api_app})

    .. versionadded: 1.0.0
    """
    def _create_app(**kwargs):
        for k in ('instance_path', 'root_path', 'static_folder'):
            if k in app_kwargs and callable(app_kwargs[k]):
                app_kwargs[k] = app_kwargs[k]()

        app = base_app(app_name, **app_kwargs)
        app_created.send(_create_app, app=app)

        debug = kwargs.get('debug')
        if debug is not None:
            app.debug = debug

        # Load configuration
        if config_loader:
            config_loader(app, **kwargs)

        # Load URL converters.
        converter_loader(
            app,
            entry_points=converter_entry_points,
            modules=converters,
        )

        # Load application based on entrypoints.
        app_loader(
            app,
            entry_points=extension_entry_points,
            modules=extensions,
        )

        # Load blueprints
        blueprint_loader(
            app,
            entry_points=blueprint_entry_points,
            modules=blueprints,
        )

        app_loaded.send(_create_app, app=app)

        # Replace WSGI application using factory if provided (e.g. to install
        # WSGI middleware).
        if wsgi_factory:
            app.wsgi_app = wsgi_factory(app, **kwargs)

        return app

    return _create_app


def create_cli(create_app=None):
    """Create CLI for ``inveniomanage`` command.

    :param create_app: Flask application factory.
    :returns: Click command group.

    .. versionadded: 1.0.0
    """
    def create_cli_app(info):
        """Application factory for CLI app.

        Internal function for creating the CLI. When invoked via
        ``inveniomanage`` FLASK_APP must be set.
        """
        if create_app is None:
            # Fallback to normal Flask behavior
            info.create_app = None
            app = info.load_app()
        else:
            app = create_app(debug=get_debug_flag())
        return app

    @click.group(cls=FlaskGroup, create_app=create_cli_app)
    def cli(**params):
        """Command Line Interface for Invenio."""
        pass

    return cli


def app_loader(app, entry_points=None, modules=None):
    """Run default application loader.

    :param entry_points: List of entry points providing to Flask extensions.
    :param modules: List of Flask extensions.

    .. versionadded: 1.0.0
    """
    _loader(app, lambda ext: ext(app), entry_points=entry_points,
            modules=modules)


def blueprint_loader(app, entry_points=None, modules=None):
    """Run default blueprint loader.

    The value of any entry_point or module passed can be either an instance of
    ``flask.Blueprint`` or a callable accepting a ``flask.Flask`` application
    instance as a single argument and returning an instance of
    ``flask.Blueprint``.

    :param entry_points: List of entry points providing to Blueprints.
    :param modules: List of Blueprints.

    .. versionadded: 1.0.0
    """
    url_prefixes = app.config.get('BLUEPRINTS_URL_PREFIXES', {})

    def loader_init_func(bp_or_func):
        bp = bp_or_func(app) if callable(bp_or_func) else bp_or_func
        app.register_blueprint(bp, url_prefix=url_prefixes.get(bp.name))

    _loader(app, loader_init_func, entry_points=entry_points, modules=modules)


def converter_loader(app, entry_points=None, modules=None):
    """Run default converter loader.

    :param entry_points: List of entry points providing to Blue.
    :param modules: Map of coverters.

    .. versionadded: 1.0.0
    """
    if entry_points:
        for entry_point in entry_points:
            for ep in pkg_resources.iter_entry_points(entry_point):
                try:
                    app.url_map.converters[ep.name] = ep.load()
                except Exception:
                    app.logger.error(
                        'Failed to initialize entry point: {0}'.format(ep))
                    raise

    if modules:
        app.url_map.converters.update(**modules)


def _loader(app, init_func, entry_points=None, modules=None):
    """Run generic loader.

    Used to load and initialize entry points and modules using an custom
    initialization function.

    .. versionadded: 1.0.0
    """
    if entry_points:
        for entry_point in entry_points:
            for ep in pkg_resources.iter_entry_points(entry_point):
                try:
                    init_func(ep.load())
                except Exception:
                    app.logger.error(
                        'Failed to initialize entry point: {0}'.format(ep))
                    raise
    if modules:
        for m in modules:
            try:
                init_func(m)
            except Exception:
                app.logger.error('Failed to initialize module: {0}'.format(m))
                raise


def base_app(import_name, instance_path=None, static_folder=None,
             static_url_path='/static', template_folder='templates',
             instance_relative_config=True, root_path=None, app_class=Flask):
    """Invenio base application factory.

    If the instance folder does not exists, it will be created.

    :param import_name: The name of the application package.
    :param env_prefix: Environment variable prefix.
    :param instance_path: Instance path for Flask application.
    :param static_folder: Static folder path.
    :param app_class: Flask application class.
    :returns: Flask application instance.

    .. versionadded: 1.0.0
    """
    configure_warnings()

    # Create the Flask application instance
    app = app_class(
        import_name,
        instance_path=instance_path,
        instance_relative_config=instance_relative_config,
        static_folder=static_folder,
        static_url_path=static_url_path,
        template_folder=template_folder,
        root_path=root_path,
    )

    # Create instance path if it doesn't exists
    try:
        if instance_path and not os.path.exists(instance_path):
            os.makedirs(instance_path)
    except Exception:  # pragma: no cover
        app.logger.exception(
            'Failed to create instance folder: "{0}"'.format(instance_path)
        )

    return app


def configure_warnings():
    """Configure warnings by routing warnings to the logging system.

    It also unhides ``DeprecationWarning``.

    .. versionadded: 1.0.0
    """
    if not sys.warnoptions:
        # Route warnings through python logging
        logging.captureWarnings(True)

        # DeprecationWarning is by default hidden, hence we force the
        # 'default' behavior on deprecation warnings which is not to hide
        # errors.
        warnings.simplefilter('default', DeprecationWarning)
        warnings.simplefilter('ignore', PendingDeprecationWarning)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""WSGI application factory for Invenio."""

from __future__ import absolute_import, print_function

import warnings

# They were moved in the same version so they can be in one try/except
try:
    from werkzeug.middleware.dispatcher import DispatcherMiddleware
    from werkzeug.middleware.proxy_fix import ProxyFix
    WERKZEUG_GTE_014 = False
except ImportError:
    from werkzeug.contrib.fixers import ProxyFix
    from werkzeug.wsgi import DispatcherMiddleware
    WERKZEUG_GTE_014 = True


def create_wsgi_factory(mounts_factories):
    """Create a WSGI application factory.

    Usage example:

    .. code-block:: python

       wsgi_factory = create_wsgi_factory({'/api': create_api})

    :param mounts_factories: Dictionary of mount points per application
        factory.

    .. versionadded:: 1.0.0
    """
    def create_wsgi(app, **kwargs):
        mounts = {
            mount: factory(**kwargs)
            for mount, factory in mounts_factories.items()
        }
        return DispatcherMiddleware(app.wsgi_app, mounts)
    return create_wsgi


def wsgi_proxyfix(factory=None):
    """Fix Flask environment according to ``X-Forwarded-_`` headers.

    .. note::

       You must set ``WSGI_PROXIES`` to the correct number of proxies,
       otherwise you application is susceptible to malicious attacks.

    .. versionadded:: 1.0.0

    .. note::

       ``PROXY_CONFIG`` lets you specify the number of proxies which a HTTP
       request is traversing before reaching the Invenio application. Invenio
       will then be able to trust and therefore pick the correct headers set
       by your proxy, for example host, protocol or scheme. This is important
       to avoid any possible malicious attacks which could manipulate/inject
       headers:

       .. code-block:: python

          PROXYFIX_CONFIG = {
              x_for: 1,  # Invenio will trust only the first value of
                         # X-Forwarded-For header and ignore the rest
              x_proto: 0,  # No X-Forwarded-Proto header will be trusted
              x_host: 0,  # No X-Forwarded-Host header will be trusted
              x_port: 0,  # No X-Forwarded-Port header will be trusted
              x_prefix: 0,  # No X-Forwarded-Prefix header will be trusted
          }

       This configuration defines which HTTP headers will be taken into
       account by your application.

       For example, if you have one proxy in front of your application and you
       want your application to use the original HTTP scheme, you would set
       the following configuration:

       .. code-block:: python

          PROXYFIX_CONFIG = {
              x_proto: 1,
          }

       Which means, that the first ``X-Forwarded-Proto`` header will be taken
       into account.

    .. versionadded:: 1.2.0

    .. deprecated:: 1.2

       The ``WSGI_PROXIES`` configuration is deprecated and it will be removed,
       use ``PROXYFIX_CONFIG`` instead.
    """
    def create_wsgi(app, **kwargs):
        wsgi_app = factory(app, **kwargs) if factory else app.wsgi_app
        num_proxies = app.config.get('WSGI_PROXIES')
        proxy_config = app.config.get('PROXYFIX_CONFIG')
        if proxy_config and not WERKZEUG_GTE_014:
            return ProxyFix(wsgi_app, **proxy_config)
        elif num_proxies:
            warnings.warn('The WSGI_PROXIES configuration is deprecated and '
                          'it will be removed, use PROXYFIX_CONFIG instead',
                          PendingDeprecationWarning)
            if WERKZEUG_GTE_014:
                return ProxyFix(wsgi_app, num_proxies=num_proxies)
            else:
                return ProxyFix(wsgi_app, x_for=num_proxies)
        return wsgi_app
    return create_wsgi
